#!/bin/sh
#
#   Module name 		: install.sh 
#   Date of last delta		: 9/16/8
#   Release number		: 1.43
#

# RedHat Linux BASH is not compiled USG, this causes that the \escape
# sequences in echo to fail using the builtin echo.
# The /bin/echo is only OK for V6.0, not V6.1 so use function to enable the
# \escape sequences (echo -e) of the bash echo.
# Also remove aliases that may cause problems.
#
os=`uname -s`
if [ "x${os}" = "xLinux" ]
then
    echotest=`echo "Test\tTest"`
    if [ "${echotest}" = "Test\\tTest" ]
    then
	echo()
	{
	    /bin/echo -e $*
	    return $?
	}
    fi
    unalias ls 2>/dev/null
    unalias rm 2>/dev/null
    unalias mv 2>/dev/null
    unalias cp 2>/dev/null
    unalias ps 2>/dev/null
fi


# Finds a free socket port number that the server can use and records it in
# the installation data file, 'install.data'.
FindPort()
{
    echo Testing for a free socket port...

    count=0
    # find a free port
    port=`$path/bin/freeport`
    
    # record the port number in the installation data file
    echo PORT=$port >> $path/install.data

    # let the user know the chosen port number
    echo "\nThe port number has now been determined and is: $port"
    echo "\nRecord this in your user documentation, as you will need to know this"
    echo "port number when you install the Client software."
    echo "\nIf you forget this number you can find it in the file 'install.data'"
    echo "in the base directory of your Server installation, by typing"
    echo "\n\tcat install.data"
    echo "\nPress [RETURN] to continue. \c"
    read trash
}


# Completes the start/stop scripts for the server setting the path
# for $IniFiles, $ServerName and freeport to $path/bin.
ModifyScript()
{
	File=$1

	# check if we have already changed it from '.'
	x=`grep "^[A-Z]*_BIN_DIR=\.$" < $path/bin/$File`

	if [ "$x" != "" ]
	then
		sed "s%^\([A-Z]*_BIN_DIR\)=.*%\1=$path/bin%" < \
			$path/bin/$File > /tmp/install.sed.$$
		if [ -s /tmp/install.sed.$$ ]
		then
			mv -f /tmp/install.sed.$$ $path/bin/$File
			chmod a+x $path/bin/$File 2>/dev/null
		else
			echo "\nFailed to modify $File\n"
			rm -f /tmp/install.sed.$$
		fi
	fi
}

# Sets up the automatic starting of the server at system boot time
EnableAutoStart()
{
    AutoStart=false
    # copy the start up script so it's run at init state 2
    if [ -d /etc/rc2.d ]
    then
    	(
	cd /etc/rc2.d
	if [ -f S99local ]
	then
	  # Must be Red Hat 7.x or later.  Normal stuff don't work.
	  echo "(cd $path/bin;./tcf_start.sh -f)" >> S99local
	else
    	  cp $path/bin/$StartScript S99$ProdNameUpper
	fi
	)
    else
	cp $path/bin/$StartScript /etc/S99$ProdNameUpper
	if [ "`grep $ProdName: /etc/inittab`" = "" ]
	then
		echo "$ProdName:2345:wait:sh /etc/S99$ProdNameUpper > /dev/null 2>&1" >> /etc/inittab
	fi
    fi

    AutoStart=true
}


# Checks that the user is the superuser
SuCheck()
{
	#If $proc = 0 then Superuser.       
	proc=`id | awk -F= '{ print $2 }' | awk -F\( '{ print $1 }'`

	if [ "$proc" != "0" ]
	then
		echo
		echo "WARNING: you are not root.  You need root access to enable the automatic start"
		echo "of the server at boot time."
		echo
		echo "Press RETURN to continue or 'x' to exit."
		read ans
		if [ "$ans" = "x" ]
		then
			exit 1
		fi
		# allow script to run without being root
		# for safe testing.
		MustBeRoot=0
	fi     
}   


CheckServerRunning()
{
    for PS_OPTION in -e ax -ax			# System V, BSD, hybrid.
    do
	ps $PS_OPTION 2>/dev/null | grep $ServerName | grep -v 'grep' > /dev/null
	if [ $? -eq 0 ]
	then
	    echo "You are advised to close down existing users of this product"
	    echo "before installing an upgrade to the product."
	    echo "Do you wish to continue ? (y/[n]): \c"
            read answer
	    if [ "$answer" != "y" -a "$answer" != "Y" ]
            then 
                echo "\nYou can re-run install.sh when product not in use" 
    	        exit
            fi
	fi

    done

}


# save the ini file
PreserveIni()
{
	Ret=0
	for file in $PreserveFiles
	do
		if [ -f $path/$file ]
		then
			if [ ! -d $PreserveDir ]
			then
				mkdir $PreserveDir
			fi
			cp $path/$file $PreserveDir
			Ret=1
		fi
	done
	echo $Ret
}

RestoreIni()
{
	Fail=0
	for file in $PreserveFiles
	do
		name=`basename $file`
		mv $PreserveDir/$name $path/$file
		if [ $? -ne 0 ]
		then
			Fail=1
		fi
	done
	if [ $Fail -ne 0 ]
	then
		echo Unable to restore ini files from $PreserveDir
	else
		rm -rf $PreserveDir
	fi
}

# get rid of ^M's
dos2unix()
{
	tr -d '\015' < $1 > /tmp/$1.tmp.$$
	mv -f /tmp/$1.tmp.$$ $1
}

InitialSetup()
{
# Insert tempdir, host name, port number, and location of message and log
# files into $IniFiles.

	if [ $ServerProd -eq 1 ]
	then
		if [ -x $path/bin/freeport ]
		then
			PORT=`$path/bin/freeport`
		else
			PORT=''
			echo "Cannot find 'freeport' executable."
		fi
	fi

	if [ $LicensedProd -eq 1 ]
	then
		origDir=`pwd`
		cd $IniFileDir
		for IniFile in $IniFiles
		do
			sed "s?License=.*?License=${path}/bin/SERVER.LIC?" \
				$IniFile > IniFile.i$$
			if [ "$DEFAULTLOG" = "" ]
			then
				cp IniFile.i$$ $IniFile
			else
				sed "s?ServerLog=.*?ServerLog=${path}/bin/$DEFAULTLOG?" IniFile.i$$ > $IniFile
			fi
			rm -f IniFile.i$$
			if [ $SetTempDir -eq 1 ]
			then
				echo "TmpDir=$path/bin" >> $IniFile
			fi
		done
		cd $origDir
	fi

	for IniFile in $IniFiles
	do
		if [ `grep ODBC_driver $IniFileDir/$IniFile`x != "x" ]
		then
			origDir=`pwd`
			cd $IniFileDir
			mv -f $IniFile IniFile.i$$
			OrODBC=`echo ${path}/lib/liborodbc.s?`
			sed "s%ODBC_driver=.*%ODBC_driver=$OrODBC%" \
				IniFile.i$$ > $IniFile
			rm -f IniFile.i$$
			cd $origDir
		fi
	done

	if [ -d SysSDLFiles ]
	then
		cd SysSDLFiles
		# These two are stored in DOS format
		dos2unix monitor.sdl
		dos2unix cfaupdate.sdl
		cd ..
	fi
}

# Note that the $ZipName[0-9].z files are written on the discs in reverse order
# So the first .z file extracted will have the highest number and the last
# disc will contain ${ZipName}1.z.

special=0

CheckExtractComplete()
{
	fname=`/bin/ls *.z 2> /dev/null | sed -n "1p"`
	if [ "$fname" = $ZipName".z" ]
	then
		# no splitting of discs has occured
		return 0
	fi
	lastfile=`/bin/ls *.z 2>/dev/null | tail -1`
	if [ "$lastfile" = "" ]
	then
		# This can happen if disc 1 just has install.sh on it, 2nd
		# disc has compressed file on it.
		echo "\nDisk 2 has not been loaded."
		special=1
		return 2
	fi
	ndiscs=`echo $lastfile | sed "s/$ZipName//;s/\.z$//"`
	count=$ndiscs
	fail=0
	while [ $count -ne 0 ]
	do
		if [ ! -f $ZipName$count.z ]
		then
			miss=`expr $ndiscs + 1 - $count + $special`
			echo "\nDisk $miss has not been loaded."
			if [ "$fail" -eq 0 ]
			then
				fail=$miss
			fi
		fi
		count=`expr $count - 1`
	done
	if [ $fail -ne 0 ]
	then
		echo "\nExtract of media not complete."
		return $fail
	fi
	return 0
}

GetDiscList()
{
	x=`tar 2>&1 | grep dev/rfd`
	if [ "$x" != "" ]
	then
		# must be on SCO or another platform that lists devices
		tar 2>&1 | awk '{ print $2 }' | grep fd
	fi
	ls /dev/fd[0-9] 2>/dev/null		# most unix's
	ls /dev/rdsk/c2d[0-9]s2 2>/dev/null	# HP unix
	ls /dev/rfd[0-9]c 2>/dev/null		# OSF 1
	ls /vol/dev/rdiskette0/* 2>/dev/null	# Sun OS
	ls /dev/rdsk/f03ht 2>/dev/null		# SV - Unixware etc.
}

# Note that the list is numbered from 0 to make the numbering compatible
# with the SCO unix tar command that lists from 0.
ChooseDevice()
{
	Tfile=/tmp/disc.list.$$
	GetDiscList > $Tfile
	DevNam="none"

	while [ ! -b $DevNam -a ! -c $DevNam ]
	do
		if [ "$DevNam" != "none" ]
		then
			echo "\n'$DevNam' is invalid, try again.\n"
		fi
		echo "\nWhat device do you want to extract from?\n"
		echo "Enter a number or a device name.\n"
		Count=-1
		for i in `cat $Tfile`
		do
			Count=`expr $Count + 1`
			echo "\t\t"$Count")\t"$i
		done

		echo "\nDevice? \c"
		read DevNam
		if [ "$DevNam" = "" ]
		then
			DevNam="none"
		else
			case $DevNam in
			[0-9]* )
				if [ $DevNam -ge 0 -a $DevNam -le $Count ]
				then
				   position=`expr $DevNam + 1`
				   DevNam=`cat $Tfile | sed -n "${position}p"`
				fi
				;;
			/* )
				;;
			* )	# Invalid option
				DevNam="invalid"
				;;
			esac
		fi
	done
	rm $Tfile
	echo $DevNam > .disc_dev
}

LoadADisc()
{
	if [ -f /bin/volcheck ]
	then
		# for Sun
		Sun=/bin/volcheck
		$Sun
		Eject="eject floppy"
	fi

	if [ ! -f .disc_dev ]
	then
		ChooseDevice
	fi
	echo "\nAbout to read from disk device "`cat .disc_dev`
	echo "\nInsert disk "$1" into drive and press"
	echo "\t'c' to change device name"
	echo "\t'q' to quit"
	echo "or\tRETURN to load disk"
	echo "\n> \c"
	read ans
	if [ "$ans" = "c" ]
	then
		ChooseDevice
	fi
	if [ "$ans" = "q" ]
	then
		echo "\nInstall aborted."
		exit 1
	fi
	if [ "$Sun" != "" ]
	then
		$Sun
	fi
	echo "\nExecuting  tar -xvf "`cat .disc_dev`
	tar -xvf `cat .disc_dev`
	Ret=$?
	if [ "$Eject" != "" ]
	then
		$Eject
	fi
	return $Ret
}

# New way of changing permissions does not effect any files that are
# already on the system.  Relies on new checksum file.
CorrectPerms()
{
	# Change ownership to root
	echo "\nChanging ownership of files..."

	if [ -d "example" ]
	then
		find example -type d -exec chmod 777 {} \; 2>/dev/null
	fi
	count=`wc -l < .checksums | sed "s/ *//g"`
	while [ $count -ne 0 ]
	do
		line=`sed -n ${count}p < .checksums`
		file=`echo $line | cut -d' ' -f1`

		# allow for README and install.sh that end up
		# in base directory.
		if [ ! -f $file ]
		then
			file=`basename $file`
		fi

		if [ $MustBeRoot -eq 1 ]
		then
			chown 0 $file
			chgrp 0 $file
		fi

		case $file in
			*.ini|*_ini* )		chmod 644 $file;;
			*install.sh* )		chmod 755 $file;;
			bin/* )			chmod 755 $file;;
			*/*.sl | */*.so )	chmod 755 $file;;
			example/* )		chmod 666 $file;;
			*/*.sh )		chmod 755 $file;;
			*/*.o | */*.a )		chmod 666 $file;;
			* )			chmod 644 $file;
		esac

		chmod 777 `dirname $file`
		if [ $MustBeRoot -eq 1 ]
		then
			chown 0 `dirname $file`
			chgrp 0 `dirname $file`
		fi

		count=`expr $count - 1`
	done
}

ExtractFiles()
{
	finished=0
	while [ $finished -eq 0 ]
	do
		CheckExtractComplete
		MissDisc=$?
		if [ $MissDisc -eq 0 ]
		then
			finished=1
		else
			LoadADisc $MissDisc
		fi
	done

	if [ -f kript.u ]
	then
	echo "\nExtracting program to decode using a product key ..."
		uudecode kript.u
	fi

	echo "\nUncompressing the tar file..."
	if [ -f ${ZipName}1.z ]
	then
		if [ -f ${ZipName}10.z ]
		then
			cat $ZipName[0-9].z ${ZipName}1[0-9].z | DoUncomp | tar -xvf -
		else
			cat $ZipName[0-9].z | DoUncomp | tar -xvf -
		fi
	else
                DoUncomp < $ZipName.z > /dev/null 2> /tmp/install.er.$$
                if [ $? -ne 0 -o -s /tmp/install.er.$$ ]
                then
    			rm -f /tmp/install.er.$$
			if [ -f ./kript ]
			then
                        DispGetPasswd
                        ./kript $answer < $ZipName.z | DoUncomp | tar -xvf -
			fi
                else
    			rm -f /tmp/install.er.$$
                        DoUncomp < $ZipName.z | tar -xvf -
                fi
	fi

	if [ $? -ne 0 -o ! -d bin ]
	then
		if [ "$Tanswer" = "$answer" ]
		then
			echo "\nUncompress failed... check for password error\n"
			exit
		else
			echo "\nUncompress failed...\n"
			exit
		fi
	fi
# Now delete the TAR file(s)
	rm -f $ZipName[0-9].z $ZipName.z ${ZipName}1[0-9].z

	if [ -f readme ]
	then
		mv readme README
	fi

	CorrectPerms

	VerifyChecksums

	if [ -f bin/licgen -a z$LicGenName != "z" ]
	then
		mv bin/licgen bin/$LicGenName
	fi


	InitialSetup

}

DispGetPasswd()
{
        answer=""
        while [ "$answer" = "" ]
        do
	echo "\nYou need the product password obtained from either the shipment"
	echo "form or your supplier(this is not the product licence)."
        echo "Enter 'q' to quit - you can then install the product later"
        echo "\nPlease enter the product password: \c"
        read answer
	if [ "$answer" = "q" -o "$answer" = "'q'" -o "$answer" = "Q" ]
	then
		echo "\nInstall aborted."
		exit 1
	fi
        Tanswer=`echo "$answer" | cut -f3 -d\- `
        if [ "$Tanswer" != "" ]
        then
                answer=$Tanswer
        fi
        done
}

rc2Check()
{
	echo "\nDo you wish to have the $FullProdName server start"
	echo "at system boot time? (y/[n]): \c"
	read answer
	if [ "$answer" = "y" -o "$answer" = "Y" ]
	then
		EnableAutoStart
		echo "\nAutostart has been set up.  Press [RETURN] to continue. \c"
		read trash
		if [ "$AutoStart" = "true" ]
		then
			echo "Auto start set up - OK" >> $path/install.data
		fi
	else
		echo "Auto start was not set up - OK" >> $path/install.data
		echo "\nRun '$PrintStartLinkScript' to start the Server after a re-boot."
		echo "\nPress [RETURN] to continue. \c"
		read trash
	fi
}

GetSumOpt()
{
	# Test for sum command and set up option variable accordingly
	# On linux:    SYS V sum requires an -s option
	# On AIX/OSF1: SYS V sum requires an -o option
	#
	echo "This is a sum test" >/tmp/$$.sumtest
	for sumopt in '-s' '-o' 'end'; do
	    if [ "$sumopt" = "end" ]; then
		sumopt=''
	    fi
	    val=`sum $sumopt /tmp/$$.sumtest 2>/dev/null`
	    ret=$?
	    if [ $ret -eq 0 ]; then
		val=`echo "$val" | awk '{ print $1 }'`
		if [ $val -eq 1652 ]; then
		    break
		fi
	    fi
	done
	rm -f /tmp/$$.sumtest
	if [ $ret -ne 0 ]; then
	    sumopt="fail"
	fi
}

# Note that there are now two checksum files.  The old one .chksums and the
# new one called .checksums.   The new one has the directory name for the
# file which is safer than doing 'ls */filename' since user could have 
# existing directories with backup versions in (could find more than one).
VerifyChecksums()
{
	if [ -f .chksums ]
	then
		GetSumOpt
		if [ "$sumopt" = "fail" ]
		then
			echo "\nNo valid 'sum' command available, skipping checks...\n"
			return
		fi
		fail=0

		echo "\nVerifying Software Installation Components' Checksums...\n"
		count=`wc -l < .chksums | sed "s/ *//g"`
		while [ $count -ne 0 ]
		do
			if [ -f .checksums ]
			then
				line=`sed -n ${count}p < .checksums`
			else
				line=`sed -n ${count}p < .chksums`
			fi
			file=`echo $line | cut -d' ' -f1`
			chks=`echo $line | cut -d' ' -f2`
			if [ -f .checksums ]
			then
				# install.sh does not get into 'bin' directory
				if [ ! -f $file ]
				then
					file=`basename $file`
				fi
			else
				if [ ! -f $file ]
				then
					file=`/bin/ls */$file`
				fi
			fi
			echo "Verifying $file ..."
			if [ "$chks" != "`sum $sumopt < $file | cut -d' ' -f1`" ]
			then
				echo "Checksum does not match for "$file
				echo "Checksum does not match for "$file >> install.data
				fail=1
			fi
			count=`expr $count - 1`
		done
		if [ $fail -eq 1 ]
		then
			ans="x"
			while [ "$ans" != "c" -a "$ans" != "C" \
				-a "$ans" != "e" -a "$ans" != "E" ]
			do
				echo "\nWARNING: All checksums did not verify correctly."
				echo "This is a corrupt release. Contact your supplier."
				echo "\nPress 'C' to continue or 'E' to exit > \c"
				read ans
				if [ "$ans" = "e" -o "$ans" = "E" ]
				then
					exit 1
				fi
			done
		else
			echo "\nAll checksums verified correctly."
			echo "Checksums verified - OK" >> install.data
		fi
	fi
}

ServerProdSetup()
{
	cd $path/bin

	# run through the initial set up procedure
	echo
	echo "Running through the set up procedures..."
	echo

	if [ $FindAPort -eq 1 ]
	then
		FindPort
		echo "Usable socket port found - OK" >> $path/install.data
	fi

	ModifyScript $StartScript
	ModifyScript $StopScript
	echo "Server start script written - OK" >> $path/install.data

	if [ "$StartLinkScripts" != "" ]
	then
		for StartLinkScript in $StartLinkScripts
		do
			rm -f $StartLinkScript
			ln -s $StartScript $StartLinkScript
		done
	fi
	if [ "$StopLinkScripts" != "" ]
	then
		for StopLinkScript in $StopLinkScripts
		do
			rm -f $StopLinkScript
			ln -s $StopScript $StopLinkScript
		done
	fi

	# check for auto start of $ServerName at boot-up option
	if [ $MustBeRoot -eq 1 ]
	then
		rc2Check
	else
		echo
		echo "Not root. Avoiding option to start at boot time."
		echo
	fi
	cd $path
}

SetUpVars()
{
	LicensedProd=1
	MustBeRoot=0
	ServerProd=0
	IniFile2=""
	ServerName="noserver"
	MkDirs=""
	LicGenName=""
	StartScript=serv_start.sh
	StopScript=serv_stop.sh
	StartLinkScripts=""
	StopLinkScripts=""
	FindAPort=1
	SetTempDir=0

	PreserveDir=/tmp/saveini.$$

	fname=`/bin/ls *.z 2> /dev/null | sed -n "1p"`
	case $fname in
	ucf* )
		ZipName=ucf
		FullProdName="Transoft Component Adapters"
		ProdName=tcf
		CobLinkProd=tca
		ProdNameUpper=TCF
		ServerProd=1
		IniFileDir=bin
		IniFiles=appserver.ini
		PreserveFiles="bin/$IniFiles bin/tcf_ini.sh bin/iodbc.ini"
		ServerName=cfaappserver
		InstName="Server "
		MkDirs="log autoupdate SysSDLFiles UserSDLFiles SAMFiles"
		MustBeRoot=1
		StartLinkScripts="tcf_start.sh"
		StopLinkScripts="tcf_stop.sh"
		PrintStartLinkScript=$StartLinkScripts
		DEFAULTLOG=appserver.log
		SetTempDir=1
		;;
	hp3* )
		ZipName=hp3
		FullProdName="Transoft HP3000 Product Suite"
		CobLinkProd=hp3
		ProdName=hp3
		ProdNameUpper=HP3
		ServerProd=1
		IniFileDir=./
		IniFiles="tga.ini tdam.ini tml.ini"
		PreserveFiles=$IniFiles
		LicGenName=hp3licgen
		FindAPort=0
		MustBeRoot=1
		StartLinkScripts="tga_start.sh tdam_start.sh tml_start.sh"
		StopLinkScripts="tga_stop.sh tdam_stop.sh tml_stop.sh"
		PrintStartLinkScript=start_all.sh
		;;
	esac
}

DoUncomp()
{
	if [ "x${os}" = "xLinux" ]
	then
		gunzip
	else
		uncompress
	fi
}

BadDirCheck()
{
	if [ $ProdName = "hp3" -a `uname -n` != "kenux" ]
	then
		case `pwd` in
		*tml* | *TML* )
			echo Attempt to install $FullProdName in directory containing TML.
			echo This is misleading as TML is only one of the three products 
			echo in the $FullProdName
			exit 1;;
		esac
	fi
}

# ------------------------ start of the script -------------------------

clear
SetUpVars
echo "================================================"
echo $FullProdName" Installation"
echo "================================================"

BadDirCheck

# find the path of the installed software
path=`pwd`

if [ $ServerProd -eq 1 ]
then
	# check we are superuser
	SuCheck

	# Warn if server is running
	CheckServerRunning
fi

# preserve the ini file
INIFILE=`PreserveIni`

# Only uncompress the files if this is the first time in
# i.e. do the .z files exist !
ls $ZipName[0-9].z > /dev/null 2>&1
if [ ! -d bin -o -f $ZipName.z -o $? -eq 0 ]
then
	ExtractFiles
	if [ $INIFILE = "1" ]
	then
		echo "Restoring original ini files.\n"
		RestoreIni
	fi
fi


######################################################
# Do these things only once at install time ! 
#########################################################

if [ $ServerProd -eq 1 ]
then
	ServerProdSetup
fi

chmod 644 ${path}/install.data

# create directories
for dir in $MkDirs
do
	if [ ! -d $dir ]
	then
		mkdir $dir; chmod a+wrx $dir
	fi
done

# Re-name the COBOL link scripts to include product name.
if [ ! -z "$CobLinkProd" ]
then
	for i in `ls */*link*.sh | grep -v $CobLinkProd | \
			 egrep -v "initlink.sh|comlink.sh"`
	do
		newname=`echo $i | sed "s%\.sh%${CobLinkProd}.sh%"`
		rm -f $newname
		mv $i $newname
	done
fi

echo "\nThe "$InstName"installation is complete.\n"
exit
